package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	db2 "applet/app/db/gim"
	"applet/app/db/model"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"time"
)

func DouShenUserRegisterConsumeForOfficial(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>DouShenUserRegisterConsumeForOfficial>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleDouShenUserRegisterConsumeForOfficial(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleDouShenUserRegisterConsumeForOfficial(msgData []byte) error {
	//1、解析mq中queue的数据结构体
	var msg *md.DouShenUserRegisterMessageStructForOfficial
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	//2、查找用户对应im系统中的数据
	gimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}
	if gimUser == nil {
		return errors.New("用户暂未注册im系统")
	}

	//2、查找是否有群
	officialGroup, err := db.DouShenImGroupGetOneByParamsForOfficial(db.Db, 1)
	if err != nil {
		return err
	}
	if officialGroup == nil {
		groupName := "官方[1]群"
		//3、创建群
		resp, err := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).CreateGroup(utils.GetCtx("", "13231", "74802", msg.MasterId), &pb.CreateGroupReq{
			Name:         groupName,
			AvatarUrl:    "",
			Introduction: "官方群",
			Extra:        "",
			MemberIds:    []int64{gimUser.Id},
		})
		if err != nil {
			return err
		}
		//插入 dou_shen_im_group 记录
		now := time.Now()
		_, err = db.DouShenImGroupInsert(db.Db, &model.DouShenImGroup{
			Kind:       1,
			Uid:        21,
			GroupId:    int(resp.GroupId),
			IsFull:     0,
			Name:       groupName,
			CreateTime: now,
			UpdateTime: now,
		})
		if err != nil {
			return err
		}
	} else {
		//加入群
		_, err = utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(utils.GetCtx("", "-1", "74802", msg.MasterId), &pb.AddGroupMembersReq{
			GroupId: int64(officialGroup.GroupId),
			UserIds: []int64{gimUser.Id},
		})
	}
	return nil
}
